#!/bin/bash

set +B -feu -o pipefail

THUMB_WIDTH=256
THUMB_HEIGHT=224
TIMEOUT=1

read -r ROM
read -r TARGET

# In case $TARGET is not absolute, make sure thumbnails end up where they need to be
cd "$(dirname "$ROM")"

# Temporary storage
TMPDIR="$(/bin/mktemp -d)"
function cleanup {
	/bin/rm -rf -- "$TMPDIR"
}
trap cleanup EXIT

# Start Xvfb in the background
/usr/bin/mkfifo "$TMPDIR/fd"
/usr/bin/Xvfb -displayfd 3 -nolisten tcp -fbdir "$TMPDIR" 3>"$TMPDIR/fd" 2>/dev/null >/dev/null & # %1
read -r XVFB_DISPLAY <"$TMPDIR/fd"

# Silence messages about fceux or Xvfb terminating
exec 3>&2 # Keep stderr around just in case
exec 2>/dev/null

# Run fceux in the background, wait, and take a screenshot
DISPLAY=":$XVFB_DISPLAY" HOME="$TMPDIR" SDL_AUDIODRIVER=disk SDL_DISKAUDIOFILE=/dev/null /sbin/fceux --nogui 1 "$ROM" 2>/dev/null >/dev/null & # %2
FCEUX_PID="$!"
/bin/sleep "$TIMEOUT"

/usr/bin/xwdtopnm "$TMPDIR/Xvfb_screen0" 2>/dev/null | \
	/usr/bin/pnmcut 0 0 256 224 | \
	/usr/bin/pnmscale -xysize "$THUMB_WIDTH" "$THUMB_HEIGHT" | \
	/usr/bin/pnmtopng 2>/dev/null >"$TMPDIR/thumb.png"

# Clean up background tasks
kill %2 2>/dev/null >/dev/null || true
kill %1 2>/dev/null >/dev/null || true
kill -9 %2 2>/dev/null >/dev/null || true
kill -9 %1 2>/dev/null >/dev/null || true

# Exit codes: SIGTERM == 143, SIGKILL == 137
wait "$FCEUX_PID" || [ "$?" -eq 137 -o "$?" -eq 143 ]

# Copy the image back to the web server
/bin/chmod 0644 "$TMPDIR/thumb.png"
/bin/cp -a "$TMPDIR/thumb.png" "$TARGET"

echo 'OK' # Only runs if we actually get here...
