<?php

declare(strict_types=1);
session_start();

function absolute($path) {
    if (!isset($_SESSION['dir'])) die('session not ready');
    return __DIR__ . '/' . $_SESSION['dir'] . '/' . $path;
}

function mkdir_if_not_exists($dir) {
    if (!file_exists($dir)) {
        mkdir($dir) || die('could not create directory');
        chmod($dir, 0703) || die('could not set directory permissions');
    }
}

if (!isset($_SESSION['dir']) || !file_exists($_SESSION['dir'])) {
    $dir = 'files/' . bin2hex(random_bytes(32));
    mkdir_if_not_exists($dir);

    $_SESSION['dir'] = $dir;
    $_SESSION['roms'] = [];
}

chdir($_SESSION['dir']) || die('chdir');

if (isset($_FILES['rom'])) {
    if ($_FILES['rom']['error'] !== UPLOAD_ERR_OK) {
        die('failed to upload');
    }

    $name = $_FILES['rom']['name'];
    $category = $_POST['category'];

    mkdir_if_not_exists($category);

    $rom =  $category . '/' . $name;
    $thumb = $category . '/' . $name . '.png';

    move_uploaded_file($_FILES['rom']['tmp_name'], $rom) || die('failed to store uploaded file');

    $fp = stream_socket_client('tcp://127.0.0.1:1024', $errno, $errstr, 2);
    if (!$fp) {
        unlink($rom);
        die('thumbnailer agent is busy, please try again');
    }
    fwrite($fp, absolute($rom) . "\n" . absolute($thumb) . "\n");
    $status = fgets($fp);
    fclose($fp);

    if ($status !== "OK\n" || !file_exists($thumb)) {
        unlink($rom);
        die('failed to create thumbnail');
    }

    if (!array_key_exists($category, $_SESSION['roms'])) {
        $_SESSION['roms'][$category] = [];
    }

    $_SESSION['roms'][$category][$name] = [
        'file' => $rom,
        'thumb' => $thumb
    ];

    ksort($_SESSION['roms'][$category]);
    ksort($_SESSION['roms']);

    header('Location: /');
    exit();
}

?>
<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <title>Emu War</title>
        <style>* { font-family: sans-serif; }</style>
    </head>
    <body>
        <h1>Emu War</h1>

        <form method="post" enctype="multipart/form-data">
            <label for="rom">ROM:</label>
            <select name="category" id="category">
                <option>Action</option>
                <option>Adventure</option>
                <option>Fighting</option>
                <option>Platformer</option>
                <option>Puzzle</option>
                <option>RPG</option>
                <option>Racing</option>
                <option>Shooter</option>
                <option>Sports</option>
                <option selected>Other</option>
            </select>
            <input type="file" name="rom" id="rom">
            <input type="submit" value="Upload">
        </form>

        <hr>
        <h2>Your ROMs:</h2>
        <?php foreach($_SESSION['roms'] as $category => $items): ?>
        <h3><?= $category ?></h3>
        <table>
            <?php foreach($items as $name => $rom): ?>
            <tr>
                <td>
                    <img src="<?= $_SESSION['dir'] . '/' . $rom['thumb'] ?>" width="80">
                </td>
                <td>
                    <a href="<?= $_SESSION['dir'] . '/' . $rom['file'] ?>"><?= $name ?></a>
                </td>
            </tr>
            <?php endforeach; ?>
        </table>
        <?php endforeach; ?>
    </body>
